//
//  MoviePosterViewController.m
//  MoviePosterFetcher
//
//  Created by John Blanco on 4/1/12.
//  Copyright (c) 2012 Rapture In Venice. All rights reserved.
//

#import "MoviePosterViewController.h"
#import "WebServices.h"
#import "UIImageView+WebCache.h"

@implementation MoviePosterViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        services_ = [[WebServices alloc] init];
    }
    
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [MessageCenter addGlobalMessageListener:MSG_POSTER_RECEIVED target:self action:SEL(moviePosterReceived:)];
}

- (void)releaseView {
    [MessageCenter removeMessageListenersForTarget:self];    
}

- (void)viewDidUnload {
    [self releaseView];
    [super viewDidUnload];
}

- (void)dealloc {
    [self releaseView];
}

#pragma mark -

- (void)moviePosterReceived:(DispatchMessage *)msg {
    __block UIActivityIndicatorView *bLoadingView_ = loadingView_;
    
    NSString *title = [msg.userInfo objectForKey:@"title"];
    NSString *imageLocation = [msg.userInfo objectForKey:@"poster"];

    if (title && imageLocation) {
        [posterImageView_ setImageWithURL:[NSURL URLWithString:imageLocation] success:^(UIImage *image) {
            [bLoadingView_ stopAnimating];
        } failure:^(NSError *error) {
            [bLoadingView_ stopAnimating];
        }];

        [movieTitleLabel_ setText:title];
    } else {
        // error
        [loadingView_ stopAnimating];
        [posterImageView_ setImage:nil];
        [movieTitleLabel_ setText:@"search failed"];
    }
}

#pragma mark -

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
    NSString *title = searchBar.text;
    [searchBar endEditing:YES];
    
    if (!IS_EMPTY_STRING(title)) {
        [loadingView_ startAnimating];
        [services_ requestMoviePosterForTitle:searchBar.text];
    }
    
}

- (void)searchBarCancelButtonClicked:(UISearchBar *)searchBar {
    [searchBar endEditing:YES];
}

- (void)searchBarTextDidBeginEditing:(UISearchBar *)searchBar {
    [searchBar setShowsCancelButton:YES animated:YES];
}

- (void)searchBarTextDidEndEditing:(UISearchBar *)searchBar {
    [searchBar setShowsCancelButton:NO animated:YES];
}

@end
